@@ -8,8 +8,7 @@ mod utilities;
 extern crate nb;
 
 use cortex_m_rt::entry;
-use stm32h7xx_hal::hal::digital::v2::{OutputPin, ToggleableOutputPin};
-use stm32h7xx_hal::{pac, prelude::*};
+use stm32h7xx_hal::{pac, prelude::*, time::MilliSeconds};
 
 use log::info;
 
@@ -37,24 +36,24 @@ fn main() -> ! {
 
     // Configure PE1 as output.
     let mut led = gpioe.pe1.into_push_pull_output();
-    led.set_low().unwrap();
+    led.set_low();
 
     // Get the delay provider.
     let mut delay = cp.SYST.delay(ccdr.clocks);
 
     // Configure the timer.
-    let mut timer = dp.TIM2.timer(1.hz(), ccdr.peripheral.TIM2, &ccdr.clocks);
+    let mut timer = dp.TIM2.timer(1.Hz(), ccdr.peripheral.TIM2, &ccdr.clocks);
 
     loop {
         for _ in 0..5 {
             // 20ms wait with timer
-            led.toggle().unwrap();
-            timer.start(20.ms());
+            led.toggle();
+            timer.start(MilliSeconds::from_ticks(20).into_rate());
             block!(timer.wait()).ok();
 
             // Delay for 500ms. Timer must operate correctly on next
             // use.
-            led.toggle().unwrap();
+            led.toggle();
             delay.delay_ms(500_u16);
         }
     }
