@@ -70,7 +70,7 @@ impl Instance for BDMA {
 
 #[cfg(feature = "rm0455")]
 impl Instance for BDMA2 {
-    type Rec = rec::Bdma;
+    type Rec = rec::Bdma2;
 
     #[inline(always)]
     fn ptr() -> *const BDMARegisterBlock {
@@ -87,6 +87,7 @@ impl Instance for BDMA2 {
 
 /// BDMA interrupts
 #[derive(Debug, Clone, Copy)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct BdmaInterrupts {
     transfer_complete: bool,
     transfer_error: bool,
@@ -95,6 +96,7 @@ pub struct BdmaInterrupts {
 
 /// Contains the complete set of configuration for a DMA stream.
 #[derive(Debug, Default, Clone, Copy)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct BdmaConfig {
     pub(crate) priority: config::Priority,
     pub(crate) memory_increment: bool,
@@ -120,24 +122,28 @@ impl DoubleBufferedConfig for BdmaConfig {
 impl BdmaConfig {
     /// Set the priority.
     #[inline(always)]
+    #[must_use]
     pub fn priority(mut self, priority: config::Priority) -> Self {
         self.priority = priority;
         self
     }
     /// Set the memory_increment.
     #[inline(always)]
+    #[must_use]
     pub fn memory_increment(mut self, memory_increment: bool) -> Self {
         self.memory_increment = memory_increment;
         self
     }
     /// Set the peripheral_increment.
     #[inline(always)]
+    #[must_use]
     pub fn peripheral_increment(mut self, peripheral_increment: bool) -> Self {
         self.peripheral_increment = peripheral_increment;
         self
     }
     /// Set the transfer_complete_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn transfer_complete_interrupt(
         mut self,
         transfer_complete_interrupt: bool,
@@ -147,6 +153,7 @@ impl BdmaConfig {
     }
     /// Set the half_transfer_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn half_transfer_interrupt(
         mut self,
         half_transfer_interrupt: bool,
@@ -156,6 +163,7 @@ impl BdmaConfig {
     }
     /// Set the transfer_error_interrupt.
     #[inline(always)]
+    #[must_use]
     pub fn transfer_error_interrupt(
         mut self,
         transfer_error_interrupt: bool,
@@ -165,6 +173,7 @@ impl BdmaConfig {
     }
     /// Set the double_buffer.
     #[inline(always)]
+    #[must_use]
     pub fn double_buffer(mut self, double_buffer: bool) -> Self {
         self.double_buffer = double_buffer;
         self
@@ -228,7 +237,7 @@ pub struct StreamsTuple<T>(
 impl<I: Instance> StreamsTuple<I> {
     /// Splits the DMA peripheral into streams.
     pub fn new(_regs: I, prec: I::Rec) -> Self {
-        prec.enable().reset();
+        let _ = prec.enable().reset(); // drop
         Self(
             Stream0 { _dma: PhantomData },
             Stream1 { _dma: PhantomData },
@@ -617,6 +626,7 @@ macro_rules! bdma_stream {
     };
 }
 
+#[cfg(not(feature = "rm0468"))]
 bdma_stream!(
     // Note: the field names start from one, unlike the RM where they start from
     // zero. May need updating if it gets fixed upstream.
@@ -653,6 +663,42 @@ bdma_stream!(
         teif8, gif8
     ),
 );
+#[cfg(feature = "rm0468")]
+bdma_stream!(
+    // For this sub-familiy, the field names do match the RM.
+    (
+        Stream0, 0, ifcr, ctcif0, chtif0, cteif0, cgif0, isr, tcif0, htif0,
+        teif0, gif0
+    ),
+    (
+        Stream1, 1, ifcr, ctcif1, chtif1, cteif1, cgif1, isr, tcif1, htif1,
+        teif1, gif1
+    ),
+    (
+        Stream2, 2, ifcr, ctcif2, chtif2, cteif2, cgif2, isr, tcif2, htif2,
+        teif2, gif2
+    ),
+    (
+        Stream3, 3, ifcr, ctcif3, chtif3, cteif3, cgif3, isr, tcif3, htif3,
+        teif3, gif3
+    ),
+    (
+        Stream4, 4, ifcr, ctcif4, chtif4, cteif4, cgif4, isr, tcif4, htif4,
+        teif4, gif4
+    ),
+    (
+        Stream5, 5, ifcr, ctcif5, chtif5, cteif5, cgif5, isr, tcif5, htif5,
+        teif5, gif5
+    ),
+    (
+        Stream6, 6, ifcr, ctcif6, chtif6, cteif6, cgif6, isr, tcif6, htif6,
+        teif6, gif6
+    ),
+    (
+        Stream7, 7, ifcr, ctcif7, chtif7, cteif7, cgif7, isr, tcif7, htif7,
+        teif7, gif7
+    ),
+);
 
 /// Type alias for the DMA Request Multiplexer
 ///
@@ -672,10 +718,8 @@ peripheral_target_address!(
         DMAReq::SPI6_RX_DMA,
         DMAReq::SPI6_TX_DMA
     ),
-    (pac::I2C4, rxdr, u8, P2M, DMAReq::I2C4_RX_DMA),
-    (pac::I2C4, txdr, u8, M2P, DMAReq::I2C4_TX_DMA),
-    (INNER: I2c<pac::I2C4>, rxdr, u8, P2M, DMAReq::I2C4_RX_DMA),
-    (INNER: I2c<pac::I2C4>, txdr, u8, M2P, DMAReq::I2C4_TX_DMA),
+    (HAL: I2c<pac::I2C4>, rxdr, u8, P2M, DMAReq::I2C4_RX_DMA),
+    (HAL: I2c<pac::I2C4>, txdr, u8, M2P, DMAReq::I2C4_TX_DMA),
 );
 
 #[cfg(not(feature = "rm0455"))]
