@@ -22,6 +22,8 @@
 //! [quartiq/stabilizer]: https://github.com/quartiq/stabilizer
 //! [notes]: https://github.com/quartiq/stabilizer/commit/ab1735950b2108eaa8d51eb63efadcd2e25c35c4
 
+use core::ptr;
+
 use crate::rcc::{rec, CoreClocks, ResetEnable};
 use crate::stm32;
 
@@ -32,13 +34,14 @@ use smoltcp::{
     wire::EthernetAddress,
 };
 
-use crate::ethernet::StationManagement;
+use crate::{
+    ethernet::{PinsRMII, StationManagement},
+    gpio::Speed,
+};
 
 // 6 DMAC, 6 SMAC, 4 q tag, 2 ethernet type II, 1500 ip MTU, 4 CRC, 2
 // padding
 const ETH_BUF_SIZE: usize = 1536;
-const ETH_NUM_TD: usize = 4;
-const ETH_NUM_RD: usize = 4;
 
 /// Transmit and Receive Descriptor fields
 #[allow(dead_code)]
@@ -93,13 +96,13 @@ impl TDes {
 
 /// Store a ring of TDes and associated buffers
 #[repr(C, packed)]
-struct TDesRing {
-    td: [TDes; ETH_NUM_TD],
-    tbuf: [[u32; ETH_BUF_SIZE / 4]; ETH_NUM_TD],
+struct TDesRing<const TD: usize> {
+    td: [TDes; TD],
+    tbuf: [[u32; ETH_BUF_SIZE / 4]; TD],
     tdidx: usize,
 }
 
-impl TDesRing {
+impl<const TD: usize> TDesRing<TD> {
     const fn new() -> Self {
         Self {
             td: [TDes {
@@ -107,8 +110,8 @@ impl TDesRing {
                 tdes1: 0,
                 tdes2: 0,
                 tdes3: 0,
-            }; ETH_NUM_TD],
-            tbuf: [[0; ETH_BUF_SIZE / 4]; ETH_NUM_TD],
+            }; TD],
+            tbuf: [[0; ETH_BUF_SIZE / 4]; TD],
             tdidx: 0,
         }
     }
@@ -119,8 +122,8 @@ impl TDesRing {
     /// will be stored in the descriptors, so ensure the TDesRing is
     /// not moved after initialisation.
     pub fn init(&mut self) {
-        for td in self.td.iter_mut() {
-            td.init();
+        for x in 0..TD {
+            self.td[x].init();
         }
         self.tdidx = 0;
 
@@ -129,9 +132,8 @@ impl TDesRing {
         unsafe {
             let dma = &*stm32::ETHERNET_DMA::ptr();
             dma.dmactx_dlar
-                .write(|w| w.bits(&self.td as *const _ as u32));
-            dma.dmactx_rlr
-                .write(|w| w.tdrl().bits(self.td.len() as u16 - 1));
+                .write(|w| w.bits(&self.td[0] as *const _ as u32));
+            dma.dmactx_rlr.write(|w| w.tdrl().bits(TD as u16 - 1));
             dma.dmactx_dtpr
                 .write(|w| w.bits(&self.td[0] as *const _ as u32));
         }
@@ -147,9 +149,7 @@ impl TDesRing {
         let x = self.tdidx;
         assert!(self.td[x].tdes3 & EMAC_DES3_OWN == 0); // Owned by us
 
-        // unsafe: tbuf is actually aligned, but with repr(packed) the
-        // compiler cannot infer this
-        let address = unsafe { self.tbuf[x].as_ptr() as u32 };
+        let address = ptr::addr_of!(self.tbuf[x]) as u32;
 
         // Read format
         self.td[x].tdes0 = address; // Buffer 1
@@ -166,7 +166,7 @@ impl TDesRing {
         cortex_m::asm::dsb();
 
         // Move the tail pointer (TPR) to the next descriptor
-        let x = (x + 1) % ETH_NUM_TD;
+        let x = (x + 1) % TD;
         unsafe {
             let dma = &*stm32::ETHERNET_DMA::ptr();
             dma.dmactx_dtpr
@@ -181,14 +181,15 @@ impl TDesRing {
         let x = self.tdidx;
 
         // Set address in descriptor
-        self.td[x].tdes0 = self.tbuf[x].as_ptr() as u32; // Buffer 1
+        self.td[x].tdes0 = ptr::addr_of!(self.tbuf[x]) as u32; // Buffer 1
 
         // Set length in descriptor
         let len = core::cmp::min(length, ETH_BUF_SIZE);
         self.td[x].tdes2 = (length as u32) & EMAC_TDES2_B1L;
 
-        // Return buffer slice
-        let addr = self.tbuf[x].as_ptr() as *mut _;
+        // Create a raw pointer in place without an intermediate reference. Use
+        // this to return a slice from the packed buffer
+        let addr = ptr::addr_of_mut!(self.tbuf[x]) as *mut _;
         core::slice::from_raw_parts_mut(addr, len)
     }
 }
@@ -240,13 +241,13 @@ impl RDes {
 
 /// Store a ring of RDes and associated buffers
 #[repr(C, packed)]
-struct RDesRing {
-    rd: [RDes; ETH_NUM_RD],
-    rbuf: [[u32; ETH_BUF_SIZE / 4]; ETH_NUM_RD],
+struct RDesRing<const RD: usize> {
+    rd: [RDes; RD],
+    rbuf: [[u32; ETH_BUF_SIZE / 4]; RD],
     rdidx: usize,
 }
 
-impl RDesRing {
+impl<const RD: usize> RDesRing<RD> {
     const fn new() -> Self {
         Self {
             rd: [RDes {
@@ -254,8 +255,8 @@ impl RDesRing {
                 rdes1: 0,
                 rdes2: 0,
                 rdes3: 0,
-            }; ETH_NUM_RD],
-            rbuf: [[0; ETH_BUF_SIZE / 4]; ETH_NUM_RD],
+            }; RD],
+            rbuf: [[0; ETH_BUF_SIZE / 4]; RD],
             rdidx: 0,
         }
     }
@@ -266,8 +267,8 @@ impl RDesRing {
     /// will be stored in the descriptors, so ensure the RDesRing is
     /// not moved after initialisation.
     pub fn init(&mut self) {
-        for rd in self.rd.iter_mut() {
-            rd.init();
+        for x in 0..RD {
+            self.rd[x].init();
         }
         self.rdidx = 0;
 
@@ -275,9 +276,8 @@ impl RDesRing {
         unsafe {
             let dma = &*stm32::ETHERNET_DMA::ptr();
             dma.dmacrx_dlar
-                .write(|w| w.bits(&self.rd as *const _ as u32));
-            dma.dmacrx_rlr
-                .write(|w| w.rdrl().bits(self.rd.len() as u16 - 1));
+                .write(|w| w.bits(&self.rd[0] as *const _ as u32));
+            dma.dmacrx_rlr.write(|w| w.rdrl().bits(RD as u16 - 1));
         }
 
         // Release descriptors to the DMA engine
@@ -301,9 +301,7 @@ impl RDesRing {
         let x = self.rdidx;
         assert!(self.rd[x].rdes3 & EMAC_DES3_OWN == 0); // Owned by us
 
-        // unsafe: rbuf is actually aligned, but with repr(packed) the
-        // compiler cannot infer this
-        let address = unsafe { self.rbuf[x].as_ptr() as u32 };
+        let address = ptr::addr_of!(self.rbuf[x]) as u32;
 
         // Read format
         self.rd[x].rdes0 = address; // Buffer 1
@@ -326,7 +324,7 @@ impl RDesRing {
         }
 
         // Update active descriptor
-        self.rdidx = (x + 1) % ETH_NUM_RD;
+        self.rdidx = (x + 1) % RD;
     }
 
     /// Access the buffer pointed to by the next RDes
@@ -340,7 +338,7 @@ impl RDesRing {
         let x = self.rdidx;
 
         // Write-back format
-        let addr = self.rbuf[x].as_ptr() as *mut u8;
+        let addr = ptr::addr_of!(self.rbuf[x]) as *mut u8;
         let len = (self.rd[x].rdes3 & EMAC_RDES3_PL) as usize;
 
         let len = core::cmp::min(len, ETH_BUF_SIZE);
@@ -348,24 +346,29 @@ impl RDesRing {
     }
 }
 
-pub struct DesRing {
-    tx: TDesRing,
-    rx: RDesRing,
+pub struct DesRing<const TD: usize, const RD: usize> {
+    tx: TDesRing<TD>,
+    rx: RDesRing<RD>,
 }
-impl DesRing {
-    pub const fn new() -> DesRing {
+impl<const TD: usize, const RD: usize> DesRing<TD, RD> {
+    pub const fn new() -> Self {
         DesRing {
             tx: TDesRing::new(),
             rx: RDesRing::new(),
         }
     }
 }
+impl<const TD: usize, const RD: usize> Default for DesRing<TD, RD> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
 
 ///
 /// Ethernet DMA
 ///
-pub struct EthernetDMA<'a> {
-    ring: &'a mut DesRing,
+pub struct EthernetDMA<'a, const TD: usize, const RD: usize> {
+    ring: &'a mut DesRing<TD, RD>,
     eth_dma: stm32::ETHERNET_DMA,
 }
 
@@ -384,7 +387,9 @@ pub struct EthernetMAC {
 ///
 /// Sets up the descriptor structures, sets up the peripheral
 /// clocks and GPIO configuration, and configures the ETH MAC and
-/// DMA peripherals.
+/// DMA peripherals. Automatically sets slew rate to VeryHigh.
+/// If you wish to use another configuration, please see
+/// [new_unchecked](new_unchecked).
 ///
 /// This method does not initialise the external PHY. However it does return an
 /// [EthernetMAC](EthernetMAC) which implements the
@@ -394,15 +399,53 @@ pub struct EthernetMAC {
 /// # Safety
 ///
 /// `EthernetDMA` shall not be moved as it is initialised here
-pub unsafe fn new_unchecked<'a>(
+pub fn new<'a, const TD: usize, const RD: usize>(
     eth_mac: stm32::ETHERNET_MAC,
     eth_mtl: stm32::ETHERNET_MTL,
     eth_dma: stm32::ETHERNET_DMA,
-    ring: &'a mut DesRing,
+    mut pins: impl PinsRMII,
+    ring: &'a mut DesRing<TD, RD>,
     mac_addr: EthernetAddress,
     prec: rec::Eth1Mac,
     clocks: &CoreClocks,
-) -> (EthernetDMA<'a>, EthernetMAC) {
+) -> (EthernetDMA<'a, TD, RD>, EthernetMAC) {
+    pins.set_speed(Speed::VeryHigh);
+    unsafe {
+        new_unchecked(eth_mac, eth_mtl, eth_dma, ring, mac_addr, prec, clocks)
+    }
+}
+
+/// Create and initialise the ethernet driver.
+///
+/// You must move in ETH_MAC, ETH_MTL, ETH_DMA.
+///
+/// Sets up the descriptor structures, sets up the peripheral
+/// clocks and GPIO configuration, and configures the ETH MAC and
+/// DMA peripherals.
+///
+/// This method does not initialise the external PHY. However it does return an
+/// [EthernetMAC](EthernetMAC) which implements the
+/// [StationManagement](super::StationManagement) trait. This can be used to
+/// communicate with the external PHY.
+///
+/// All the documented interrupts in the `MMC_TX_INTERRUPT_MASK` and
+/// `MMC_RX_INTERRUPT_MASK` registers are masked, since these cause unexpected
+/// interrupts after a number of days of heavy ethernet traffic. If these
+/// interrupts are desired, you can be unmask them in your own code after this
+/// method.
+///
+/// # Safety
+///
+/// `EthernetDMA` shall not be moved as it is initialised here
+pub unsafe fn new_unchecked<'a, const TD: usize, const RD: usize>(
+    eth_mac: stm32::ETHERNET_MAC,
+    eth_mtl: stm32::ETHERNET_MTL,
+    eth_dma: stm32::ETHERNET_DMA,
+    ring: &'a mut DesRing<TD, RD>,
+    mac_addr: EthernetAddress,
+    prec: rec::Eth1Mac,
+    clocks: &CoreClocks,
+) -> (EthernetDMA<'a, TD, RD>, EthernetMAC) {
     // RCC
     {
         let rcc = &*stm32::RCC::ptr();
@@ -479,7 +522,14 @@ pub unsafe fn new_unchecked<'a>(
                 .dcrcc()
                 .clear_bit()
         });
-        // Set the MAC address
+        // Set the MAC address.
+        // Writes to LR trigger both registers to be loaded into the MAC,
+        // so write to LR last.
+        eth_mac.maca0hr.write(|w| {
+            w.addrhi().bits(
+                u16::from(mac_addr.0[4]) | (u16::from(mac_addr.0[5]) << 8),
+            )
+        });
         eth_mac.maca0lr.write(|w| {
             w.addrlo().bits(
                 u32::from(mac_addr.0[0])
@@ -488,14 +538,6 @@ pub unsafe fn new_unchecked<'a>(
                     | (u32::from(mac_addr.0[3]) << 24),
             )
         });
-        eth_mac.maca0hr.write(
-            |w| {
-                w.addrhi().bits(
-                    u16::from(mac_addr.0[4]) | (u16::from(mac_addr.0[5]) << 8),
-                )
-            }, //.sa().clear_bit()
-               //.mbc().bits(0b000000)
-        );
         // frame filter register
         eth_mac.macpfr.modify(|_, w| {
             w.dntu()
@@ -524,7 +566,7 @@ pub unsafe fn new_unchecked<'a>(
                 .clear_bit()
                 // Receive All
                 .ra()
-                .set_bit()
+                .clear_bit()
                 // Promiscuous mode
                 .pr()
                 .clear_bit()
@@ -536,6 +578,44 @@ pub unsafe fn new_unchecked<'a>(
             w.pt().bits(0x100)
         });
         eth_mac.macrx_fcr.modify(|_, w| w);
+
+        // Mask away Ethernet MAC MMC RX/TX interrupts. These are statistics
+        // counter interrupts and are enabled by default. We need to manually
+        // disable various ethernet interrupts so they don't unintentionally
+        // hang the device. The user is free to re-enable them later to provide
+        // ethernet MAC-related statistics
+        eth_mac.mmc_rx_interrupt_mask.modify(|_, w| {
+            w.rxlpiuscim()
+                .set_bit()
+                .rxucgpim()
+                .set_bit()
+                .rxalgnerpim()
+                .set_bit()
+                .rxcrcerpim()
+                .set_bit()
+        });
+
+        eth_mac.mmc_tx_interrupt_mask.modify(|_, w| {
+            w.txlpiuscim()
+                .set_bit()
+                .txgpktim()
+                .set_bit()
+                .txmcolgpim()
+                .set_bit()
+                .txscolgpim()
+                .set_bit()
+        });
+        // TODO: The MMC_TX/RX_INTERRUPT_MASK registers incorrectly mark
+        // LPITRCIM as read-only, so svd2rust doens't generate bindings to
+        // modify them. Instead, as a workaround, we manually manipulate the
+        // bits
+        eth_mac
+            .mmc_tx_interrupt_mask
+            .modify(|r, w| w.bits(r.bits() | (1 << 27)));
+        eth_mac
+            .mmc_rx_interrupt_mask
+            .modify(|r, w| w.bits(r.bits() | (1 << 27)));
+
         eth_mtl.mtlrx_qomr.modify(|_, w| {
             w
                 // Receive store and forward
@@ -635,7 +715,7 @@ pub unsafe fn new_unchecked<'a>(
     // MAC layer
 
     // Set the MDC clock frequency in the range 1MHz - 2.5MHz
-    let hclk_mhz = clocks.hclk().0 / 1_000_000;
+    let hclk_mhz = clocks.hclk().raw() / 1_000_000;
     let csr_clock_range = match hclk_mhz {
         0..=34 => 2,    // Divide by 16
         35..=59 => 3,   // Divide by 26
@@ -715,9 +795,9 @@ impl StationManagement for EthernetMAC {
 }
 
 /// Define TxToken type and implement consume method
-pub struct TxToken<'a>(&'a mut TDesRing);
+pub struct TxToken<'a, const TD: usize>(&'a mut TDesRing<TD>);
 
-impl<'a> phy::TxToken for TxToken<'a> {
+impl<'a, const TD: usize> phy::TxToken for TxToken<'a, TD> {
     fn consume<R, F>(
         self,
         _timestamp: Instant,
@@ -736,9 +816,9 @@ impl<'a> phy::TxToken for TxToken<'a> {
 }
 
 /// Define RxToken type and implement consume method
-pub struct RxToken<'a>(&'a mut RDesRing);
+pub struct RxToken<'a, const RD: usize>(&'a mut RDesRing<RD>);
 
-impl<'a> phy::RxToken for RxToken<'a> {
+impl<'a, const RD: usize> phy::RxToken for RxToken<'a, RD> {
     fn consume<R, F>(self, _timestamp: Instant, f: F) -> smoltcp::Result<R>
     where
         F: FnOnce(&mut [u8]) -> smoltcp::Result<R>,
@@ -750,9 +830,11 @@ impl<'a> phy::RxToken for RxToken<'a> {
 }
 
 /// Implement the smoltcp Device interface
-impl<'a> phy::Device<'a> for EthernetDMA<'_> {
-    type RxToken = RxToken<'a>;
-    type TxToken = TxToken<'a>;
+impl<'a, const TD: usize, const RD: usize> phy::Device<'a>
+    for EthernetDMA<'_, TD, RD>
+{
+    type RxToken = RxToken<'a, RD>;
+    type TxToken = TxToken<'a, TD>;
 
     // Clippy false positive because DeviceCapabilities is non-exhaustive
     #[allow(clippy::field_reassign_with_default)]
@@ -761,11 +843,11 @@ impl<'a> phy::Device<'a> for EthernetDMA<'_> {
         // ethernet frame type II (6 smac, 6 dmac, 2 ethertype),
         // sans CRC (4), 1500 IP MTU
         caps.max_transmission_unit = 1514;
-        caps.max_burst_size = Some(core::cmp::min(ETH_NUM_TD, ETH_NUM_RD));
+        caps.max_burst_size = Some(core::cmp::min(TD, RD));
         caps
     }
 
-    fn receive(&mut self) -> Option<(RxToken, TxToken)> {
+    fn receive(&mut self) -> Option<(RxToken<RD>, TxToken<TD>)> {
         // Skip all queued packets with errors.
         while self.ring.rx.available() && !self.ring.rx.valid() {
             self.ring.rx.release()
@@ -778,7 +860,7 @@ impl<'a> phy::Device<'a> for EthernetDMA<'_> {
         }
     }
 
-    fn transmit(&mut self) -> Option<TxToken> {
+    fn transmit(&mut self) -> Option<TxToken<TD>> {
         if self.ring.tx.available() {
             Some(TxToken(&mut self.ring.tx))
         } else {
@@ -787,7 +869,7 @@ impl<'a> phy::Device<'a> for EthernetDMA<'_> {
     }
 }
 
-impl EthernetDMA<'_> {
+impl<const TD: usize, const RD: usize> EthernetDMA<'_, TD, RD> {
     /// Return the number of packets dropped since this method was
     /// last called
     pub fn number_packets_dropped(&self) -> u32 {
@@ -795,6 +877,11 @@ impl EthernetDMA<'_> {
     }
 }
 
+/// Clears the Ethernet interrupt flag
+///
+/// # Safety
+///
+/// This method implements a single register write to DMACSR
 pub unsafe fn interrupt_handler() {
     let eth_dma = &*stm32::ETHERNET_DMA::ptr();
     eth_dma
@@ -804,6 +891,15 @@ pub unsafe fn interrupt_handler() {
     let _ = eth_dma.dmacsr.read(); // Delay 2 peripheral clocks
 }
 
+/// Enables the Ethernet Interrupt. The following interrupts are enabled:
+///
+/// * Normal Interrupt `NIE`
+/// * Receive Interrupt `RIE`
+/// * Transmit Interript `TIE`
+///
+/// # Safety
+///
+/// This method implements a single RMW to DMACIER
 pub unsafe fn enable_interrupt() {
     let eth_dma = &*stm32::ETHERNET_DMA::ptr();
     eth_dma
