@@ -23,7 +23,7 @@
 //! // Constrain and Freeze power
 //! ...
 //! let rcc = dp.RCC.constrain();
-//! let ccdr = rcc.sys_ck(100.mhz()).freeze(pwrcfg, &dp.SYSCFG);
+//! let ccdr = rcc.sys_ck(100.MHz()).freeze(pwrcfg, &dp.SYSCFG);
 //!
 //! // Enable the clock to a peripheral and reset it
 //! ccdr.peripheral.FDCAN.enable().reset();
@@ -57,6 +57,17 @@
 //! // Can't set group kernel clock (it would also affect I2C3)
 //! // ccdr.peripheral.kernel_i2c123_clk_mux(I2c123ClkSel::HSI_KER);
 //! ```
+//!
+//! # REC object
+//!
+//! There is a REC object for each peripheral. For example:
+//!
+//! ```
+//! let rec_object = ccdr.peripheral.FDCAN;
+//! ```
+//!
+//! If REC object is dropped by user code, then the Reset or Enable state of
+//! this peripheral cannot be modified for the lifetime of the program.
 #![deny(missing_docs)]
 
 use core::marker::PhantomData;
@@ -68,10 +79,13 @@ use cortex_m::interrupt;
 /// A trait for Resetting, Enabling and Disabling a single peripheral
 pub trait ResetEnable {
     /// Enable this peripheral
+    #[allow(clippy::return_self_not_must_use)]
     fn enable(self) -> Self;
     /// Disable this peripheral
+    #[allow(clippy::return_self_not_must_use)]
     fn disable(self) -> Self;
     /// Reset this peripheral
+    #[allow(clippy::return_self_not_must_use)]
     fn reset(self) -> Self;
 }
 
@@ -269,6 +283,7 @@ macro_rules! peripheral_reset_and_enable_control_generator {
             $( #[ $pmeta ] )*
             impl $p {
                 /// Set Low Power Mode for peripheral
+                #[allow(clippy::return_self_not_must_use)]
                 pub fn low_power(self, lpm: LowPowerMode) -> Self {
                     // unsafe: Owned exclusive access to this bitfield
                     interrupt::free(|_| {
@@ -335,6 +350,7 @@ macro_rules! peripheral_reset_and_enable_control_generator {
             impl $p {
                 $(      // Individual kernel clocks
                     #[inline(always)]
+                    #[allow(clippy::return_self_not_must_use)]
                     /// Modify the kernel clock for
                     #[doc=$clk_doc "."]
                     /// See RM0433 Rev 7 Section 8.5.8.
@@ -439,16 +455,23 @@ macro_rules! peripheral_reset_and_enable_control_generator {
 macro_rules! variant_return_type {
     ($t:ty) => { $t };
     ($t:ty, $Variant: ident) => {
-        stm32h7::Variant<u8, $t>
+        Option<$t>
     };
 }
 
 // Register for autonomous mode enable bits
+#[cfg(not(feature = "rm0455"))]
 macro_rules! autonomous {
     ($Auto:ident) => {
         &(*RCC::ptr()).d3amr
     };
 }
+#[cfg(feature = "rm0455")]
+macro_rules! autonomous {
+    ($Auto:ident) => {
+        &(*RCC::ptr()).srdamr
+    };
+}
 
 // Enumerate all peripherals and optional clock multiplexers
 //
@@ -473,14 +496,18 @@ peripheral_reset_and_enable_control! {
     ];
     #[cfg(not(feature = "rm0455"))]
     AHB1, "" => [
-        Usb1Otg [group clk: Usb d2ccip2 "USB"],
-        Usb2Otg [group clk: Usb],
         Eth1Mac,
         #[cfg(any(feature = "rm0399"))] Art,
-        Adc12 [group clk: Adc(Variant) d3ccip "ADC"]
+        Adc12 [group clk: Adc(Variant) d3ccip "ADC"],
+        Usb1Otg [group clk: Usb d2ccip2 "USB"]
+    ];
+    #[cfg(any(feature = "rm0433", feature = "rm0399"))]
+    AHB1, "" => [
+        Usb2Otg [group clk: Usb]
     ];
     #[cfg(feature = "rm0455")]
     AHB1, "" => [
+        Crc,
         Usb1Otg [group clk: Usb cdccip2 "USB"],
         Adc12 [group clk: Adc(Variant) srdccip "ADC"]
     ];
@@ -499,43 +526,59 @@ peripheral_reset_and_enable_control! {
     AHB2, "" => [
         Rng [kernel clk: Rng cdccip2 "RNG"]
     ];
+    #[cfg(feature = "rm0468")]
+    AHB2, "" => [
+        Cordic, Fmac
+    ];
 
 
     #[cfg(all())]
     AHB3, "AMBA High-performance Bus (AHB3) peripherals" => [
-        Jpgdec, Dma2d, Mdma
+        Dma2d, Mdma
     ];
     #[cfg(not(feature = "rm0455"))]
     AHB3, "" => [
         Sdmmc1 [group clk: Sdmmc d1ccip "SDMMC"],
-        Fmc [kernel clk: Fmc d1ccip "FMC"],
+        Fmc [kernel clk: Fmc d1ccip "FMC"]
+    ];
+    #[cfg(any(feature = "rm0433", feature = "rm0399"))]
+    AHB3, "" => [
+        Jpgdec,
         Qspi [kernel clk: Qspi d1ccip "QUADSPI"]
     ];
     #[cfg(feature = "rm0455")]
     AHB3, "" => [
+        Jpgdec,
         Sdmmc1 [group clk: Sdmmc cdccip "SDMMC"],
-        Fmc [kernel clk: Fmc cdccip "FMC"]
+        Fmc [kernel clk: Fmc cdccip "FMC"],
+        Octospi1 [group clk: Octospi cdccip "OCTOSPI"],
+        Octospi2 [group clk: Octospi]
+    ];
+    #[cfg(feature = "rm0468")]
+    AHB3, "" => [
+        Octospi1 [group clk: Octospi d1ccip "OCTOSPI"],
+        Octospi2 [group clk: Octospi]
     ];
 
 
     #[cfg(all())]
     AHB4, "AMBA High-performance Bus (AHB4) peripherals" => [
-        (Auto) Bdma,
-        (Auto) Crc,
-
         Gpioa, Gpiob, Gpioc, Gpiod, Gpioe, Gpiof, Gpiog, Gpioh, Gpioi, Gpioj, Gpiok
     ];
     #[cfg(not(feature = "rm0455"))]
     AHB4, "" => [
+        (Auto) Crc,
+        (Auto) Bdma,
         (Auto) Adc3 [group clk: Adc]
     ];
+    #[cfg(feature = "rm0455")]
+    AHB4, "" => [
+        (Auto) Bdma2
+    ];
 
 
     #[cfg(all())]
     APB1L, "Advanced Peripheral Bus 1L (APB1L) peripherals" => [
-        I2c2 [group clk: I2c123],
-        I2c3 [group clk: I2c123],
-
         Spi2 [group clk: Spi123],
         Spi3 [group clk: Spi123],
 
@@ -551,20 +594,34 @@ peripheral_reset_and_enable_control! {
     APB1L, "" => [
         Dac12,
 
-        I2c1 [group clk: I2c123 d2ccip2 "I2C1/2/3"],
         Cec [kernel clk: Cec(Variant) d2ccip2 "CEC"],
         Lptim1 [kernel clk: Lptim1(Variant) d2ccip2 "LPTIM1"],
         Usart2 [group clk: Usart234578(Variant) d2ccip2 "USART2/3/4/5/7/8"]
     ];
+    #[cfg(any(feature = "rm0433", feature = "rm0399"))]
+    APB1L, "" => [
+        I2c1 [group clk: I2c123 d2ccip2 "I2C1/2/3"],
+        I2c2 [group clk: I2c123],
+        I2c3 [group clk: I2c123]
+    ];
     #[cfg(feature = "rm0455")]
     APB1L, "" => [
         Dac1,
 
         I2c1 [group clk: I2c123 cdccip2 "I2C1/2/3"],
+        I2c2 [group clk: I2c123],
+        I2c3 [group clk: I2c123],
         Cec [kernel clk: Cec(Variant) cdccip2 "CEC"],
         Lptim1 [kernel clk: Lptim1(Variant) cdccip2 "LPTIM1"],
         Usart2 [group clk: Usart234578(Variant) cdccip2 "USART2/3/4/5/7/8"]
     ];
+    #[cfg(feature = "rm0468")]
+    APB1L, "" => [
+        I2c1 [group clk: I2c1235 d2ccip2 "I2C1/2/3/5"],
+        I2c2 [group clk: I2c1235],
+        I2c3 [group clk: I2c1235],
+        I2c5 [group clk: I2c1235]
+    ];
 
 
     #[cfg(all())]
@@ -573,19 +630,25 @@ peripheral_reset_and_enable_control! {
     ];
     #[cfg(not(feature = "rm0455"))]
     APB1H, "" => [
-        Fdcan [kernel clk: Fdcan(Variant) d2ccip1 "FDCAN"],
+        Fdcan [kernel clk: Fdcan(Variant) d2ccip1 "FDCAN"]
+    ];
+    #[cfg(any(feature = "rm0433", feature = "rm0399"))]
+    APB1H, "" => [
         Swp [kernel clk: Swp d2ccip1 "SWPMI"]
     ];
     #[cfg(feature = "rm0455")]
     APB1H, "" => [
         Fdcan [kernel clk: Fdcan(Variant) cdccip1 "FDCAN"],
-        Swp [kernel clk: Swp cdccip1 "SWPMI"]
+        Swpmi [kernel clk: Swpmi cdccip1 "SWPMI"]
+    ];
+    #[cfg(feature = "rm0468")]
+    APB1H, "" => [
+        Swpmi [kernel clk: Swpmi d2ccip1 "SWPMI"]
     ];
 
 
     #[cfg(all())]
     APB2, "Advanced Peripheral Bus 2 (APB2) peripherals" => [
-        Hrtim,
         Tim1, Tim8, Tim15, Tim16, Tim17
     ];
     #[cfg(not(feature = "rm0455"))]
@@ -593,12 +656,17 @@ peripheral_reset_and_enable_control! {
         Dfsdm1 [kernel clk: Dfsdm1 d2ccip1 "DFSDM1"],
 
         Sai1 [kernel clk: Sai1(Variant) d2ccip1 "SAI1"],
-        Sai2 [group clk: Sai23(Variant) d2ccip1 "SAI2/3"],
-        Sai3 [group clk: Sai23],
 
         Spi1 [group clk: Spi123(Variant) d2ccip1 "SPI1/2/3"],
         Spi4 [group clk: Spi45(Variant) d2ccip1 "SPI4/5"],
-        Spi5 [group clk: Spi45],
+        Spi5 [group clk: Spi45]
+    ];
+    #[cfg(any(feature = "rm0433", feature = "rm0399"))]
+    APB2, "" => [
+        Hrtim,
+
+        Sai2 [group clk: Sai23(Variant) d2ccip1 "SAI2/3"],
+        Sai3 [group clk: Sai23],
 
         Usart1 [group clk: Usart16(Variant) d2ccip2 "USART1/6"],
         Usart6 [group clk: Usart16]
@@ -620,6 +688,11 @@ peripheral_reset_and_enable_control! {
         Usart1 [group clk: Usart16910(Variant) cdccip2 "USART1/6/9/10"],
         Usart6 [group clk: Usart16910]
     ];
+    #[cfg(feature = "rm0468")]
+    APB2, "" => [
+        Usart1 [group clk: Usart16910(Variant) d2ccip2 "USART1/6/9/10"],
+        Usart6 [group clk: Usart16910]
+    ];
 
 
     #[cfg(all())]
