@@ -1,4 +1,9 @@
 //! Serial
+//!
+//! # Examples
+//!
+//! - [Simple Blocking Example](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/serial.rs)
+//! - [Serial Transfer using DMA](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/serial-dma.rs)
 
 use core::fmt;
 use core::marker::PhantomData;
@@ -9,42 +14,28 @@ use embedded_hal::prelude::*;
 use embedded_hal::serial;
 use nb::block;
 
-use crate::stm32;
-use crate::stm32::usart1::cr1::{M0_A as M0, PCE_A as PCE, PS_A as PS};
-use stm32h7::Variant::Val;
+use stm32::usart1::cr2::{
+    CLKEN_A, CPHA_A, CPOL_A, LBCL_A, MSBFIRST_A, RXINV_A, TXINV_A,
+};
 
+use crate::gpio::{self, Alternate};
+use crate::rcc::{rec, CoreClocks, ResetEnable};
+use crate::stm32;
 #[cfg(feature = "rm0455")]
 use crate::stm32::rcc::cdccip2r::{USART16910SEL_A, USART234578SEL_A};
-#[cfg(not(feature = "rm0455"))]
+#[cfg(feature = "rm0468")]
+use crate::stm32::rcc::d2ccip2r::{USART16910SEL_A, USART234578SEL_A};
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 use crate::stm32::rcc::d2ccip2r::{USART16SEL_A, USART234578SEL_A};
 
+use crate::stm32::usart1::cr1::{M0_A as M0, PCE_A as PCE, PS_A as PS};
 use crate::stm32::{UART4, UART5, UART7, UART8};
 use crate::stm32::{USART1, USART2, USART3, USART6};
-
-use crate::gpio::gpioa::{
-    PA0, PA1, PA10, PA11, PA12, PA15, PA2, PA3, PA4, PA8, PA9,
-};
-use crate::gpio::gpiob::{
-    PB10, PB11, PB12, PB13, PB14, PB15, PB3, PB4, PB5, PB6, PB7, PB8, PB9,
-};
-use crate::gpio::gpioc::{PC10, PC11, PC12, PC6, PC7, PC8};
-use crate::gpio::gpiod::{PD0, PD1, PD10, PD2, PD5, PD6, PD7, PD8, PD9};
-use crate::gpio::gpioe::{PE0, PE1, PE7, PE8};
-use crate::gpio::gpiof::{PF6, PF7};
-use crate::gpio::gpiog::{PG14, PG7, PG9};
-use crate::gpio::gpioh::{PH13, PH14};
-use crate::gpio::gpioi::PI9;
-#[cfg(not(feature = "stm32h7b0"))]
-use crate::gpio::gpioj::{PJ8, PJ9};
-
-use crate::gpio::{Alternate, AF11, AF14, AF4, AF6, AF7, AF8};
-use crate::rcc::{rec, CoreClocks, ResetEnable};
 use crate::time::Hertz;
 
-use crate::Never;
-
 /// Serial error
-#[derive(Debug)]
+#[derive(Debug, Copy, Clone, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[non_exhaustive]
 pub enum Error {
     /// Framing error
@@ -59,6 +50,7 @@ pub enum Error {
 
 /// Interrupt event
 #[derive(Copy, Clone, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub enum Event {
     /// New data has been received
     Rxne,
@@ -66,24 +58,39 @@ pub enum Event {
     Txe,
     /// Idle line state detected
     Idle,
+
+    ///Tx threshlold interrupt enable
+    Txftie,
+
+    ///Rx threshlold interrupt enable
+    Rxftie,
 }
 
 pub mod config {
     use crate::time::Hertz;
 
     #[derive(Copy, Clone, PartialEq)]
-    pub enum WordLength {
-        DataBits8,
-        DataBits9,
+    pub enum FifoThreshold {
+        Eighth,
+        Quarter,
+        Half,
+        ThreeQuarter,
+        SevenEighth,
+        Full,
     }
 
+    /// The parity bits appended to each serial data word
+    ///
+    /// When enabled parity bits will be automatically added by hardware on transmit, and automatically checked by
+    /// hardware on receive. For example, `read()` would return [`Error::Parity`](super::Error::Parity).
+    ///
+    /// Note that parity bits are included in the serial word length, so if parity is used word length will be set to 9.
     #[derive(Copy, Clone, PartialEq)]
     pub enum Parity {
         ParityNone,
         ParityEven,
         ParityOdd,
     }
-
     #[derive(Copy, Clone, PartialEq)]
     pub enum StopBits {
         #[doc = "1 stop bit"]
@@ -95,17 +102,70 @@ pub mod config {
         #[doc = "1.5 stop bits"]
         STOP1P5,
     }
+    #[derive(Copy, Clone, PartialEq)]
+    pub enum BitOrder {
+        LsbFirst,
+        MsbFirst,
+    }
+    #[derive(Copy, Clone, PartialEq)]
+    pub enum ClockPhase {
+        First,
+        Second,
+    }
+    #[derive(Copy, Clone, PartialEq)]
+    pub enum ClockPolarity {
+        IdleHigh,
+        IdleLow,
+    }
 
+    /// A structure for specifying the USART or UART configuration. Fields
+    /// relating to synchronous mode are ignored for UART peripherals.
+    ///
+    /// This structure uses builder semantics to generate the configuration.
+    ///
+    /// ```
+    /// let config = Config::new().partity_odd();
+    /// ```
+    #[derive(Copy, Clone)]
     pub struct Config {
         pub baudrate: Hertz,
-        pub wordlength: WordLength,
         pub parity: Parity,
         pub stopbits: StopBits,
+        pub bitorder: BitOrder,
+        pub clockphase: ClockPhase,
+        pub clockpolarity: ClockPolarity,
+        pub lastbitclockpulse: bool,
+        pub swaptxrx: bool,
+        pub invertrx: bool,
+        pub inverttx: bool,
+        pub rxfifothreshold: FifoThreshold,
+        pub txfifothreshold: FifoThreshold,
     }
 
     impl Config {
-        pub fn baudrate(mut self, baudrate: impl Into<Hertz>) -> Self {
-            self.baudrate = baudrate.into();
+        /// Create a default configuration for the USART or UART interface
+        ///
+        /// * 8 bits, 1 stop bit, no parity (8N1)
+        /// * LSB first
+        pub fn new(frequency: Hertz) -> Self {
+            Config {
+                baudrate: frequency,
+                parity: Parity::ParityNone,
+                stopbits: StopBits::STOP1,
+                bitorder: BitOrder::LsbFirst,
+                clockphase: ClockPhase::First,
+                clockpolarity: ClockPolarity::IdleLow,
+                lastbitclockpulse: false,
+                swaptxrx: false,
+                invertrx: false,
+                inverttx: false,
+                rxfifothreshold: FifoThreshold::Eighth,
+                txfifothreshold: FifoThreshold::Eighth,
+            }
+        }
+
+        pub fn baudrate(mut self, baudrate: Hertz) -> Self {
+            self.baudrate = baudrate;
             self
         }
 
@@ -114,57 +174,106 @@ pub mod config {
             self
         }
 
+        /// Enables Even Parity
+        ///
+        /// Note that parity bits are included in the serial word length, so if parity is used word length will be set
+        /// to 9.
         pub fn parity_even(mut self) -> Self {
             self.parity = Parity::ParityEven;
             self
         }
 
+        /// Enables Odd Parity
+        ///
+        /// Note that parity bits are included in the serial word length, so if parity is used word length will be set
+        /// to 9.
         pub fn parity_odd(mut self) -> Self {
             self.parity = Parity::ParityOdd;
             self
         }
 
-        pub fn wordlength_8(mut self) -> Self {
-            self.wordlength = WordLength::DataBits8;
-            self
-        }
-
-        pub fn wordlength_9(mut self) -> Self {
-            self.wordlength = WordLength::DataBits9;
-            self
-        }
-
+        /// Specify the number of stop bits
         pub fn stopbits(mut self, stopbits: StopBits) -> Self {
             self.stopbits = stopbits;
             self
         }
+        /// Specify the bit order
+        pub fn bitorder(mut self, bitorder: BitOrder) -> Self {
+            self.bitorder = bitorder;
+            self
+        }
+        /// Specify the clock phase. Only applies to USART peripherals
+        pub fn clockphase(mut self, clockphase: ClockPhase) -> Self {
+            self.clockphase = clockphase;
+            self
+        }
+        /// Specify the clock polarity. Only applies to USART peripherals
+        pub fn clockpolarity(mut self, clockpolarity: ClockPolarity) -> Self {
+            self.clockpolarity = clockpolarity;
+            self
+        }
+        /// Specify if the last bit transmitted in each word has a corresponding
+        /// clock pulse in the SCLK pin. Only applies to USART peripherals
+        pub fn lastbitclockpulse(mut self, lastbitclockpulse: bool) -> Self {
+            self.lastbitclockpulse = lastbitclockpulse;
+            self
+        }
+
+        /// If `true`, swap the Tx and Rx pins
+        pub fn swaptxrx(mut self, swaptxrx: bool) -> Self {
+            self.swaptxrx = swaptxrx;
+            self
+        }
+
+        /// If `true`, RX pin signal levels are inverted
+        pub fn invertrx(mut self, invertrx: bool) -> Self {
+            self.invertrx = invertrx;
+            self
+        }
+
+        /// If `true`, TX pin signal levels are inverted
+        pub fn inverttx(mut self, inverttx: bool) -> Self {
+            self.inverttx = inverttx;
+            self
+        }
+
+        pub fn rxfifothreshold(
+            mut self,
+            rxfifothreshold: FifoThreshold,
+        ) -> Self {
+            self.rxfifothreshold = rxfifothreshold;
+            self
+        }
+
+        pub fn txfifothreshold(
+            mut self,
+            txfifothreshold: FifoThreshold,
+        ) -> Self {
+            self.txfifothreshold = txfifothreshold;
+            self
+        }
     }
 
     #[derive(Debug)]
+    #[cfg_attr(feature = "defmt", derive(defmt::Format))]
     pub struct InvalidConfig;
 
     impl Default for Config {
         fn default() -> Config {
-            Config {
-                baudrate: Hertz(19_200), // 19k2 baud
-                wordlength: WordLength::DataBits8,
-                parity: Parity::ParityNone,
-                stopbits: StopBits::STOP1,
-            }
+            Self::new(Hertz::from_raw(19_200)) // 19k2 baud
         }
     }
 
-    impl<T: Into<Hertz>> From<T> for Config {
-        fn from(f: T) -> Config {
-            Config {
-                baudrate: f.into(),
-                ..Default::default()
-            }
+    impl From<Hertz> for Config {
+        fn from(frequency: Hertz) -> Config {
+            Self::new(frequency)
         }
     }
 }
 
-pub trait Pins<USART> {}
+pub trait Pins<USART> {
+    const SYNCHRONOUS: bool = false;
+}
 pub trait PinTx<USART> {}
 pub trait PinRx<USART> {}
 pub trait PinCk<USART> {}
@@ -176,6 +285,15 @@ where
 {
 }
 
+impl<USART, TX, RX, CK> Pins<USART> for (TX, RX, CK)
+where
+    TX: PinTx<USART>,
+    RX: PinRx<USART>,
+    CK: PinCk<USART>,
+{
+    const SYNCHRONOUS: bool = true;
+}
+
 /// A filler type for when the Tx pin is unnecessary
 pub struct NoTx;
 /// A filler type for when the Rx pin is unnecessary
@@ -220,144 +338,147 @@ usart_pins! {
     USART1:
         TX: [
             NoTx,
-            PA9<Alternate<AF7>>,
-            PB6<Alternate<AF7>>,
-            PB14<Alternate<AF4>>
+            gpio::PA9<Alternate<7>>,
+            gpio::PB6<Alternate<7>>,
+            gpio::PB14<Alternate<4>>
         ]
         RX: [
             NoRx,
-            PA10<Alternate<AF7>>,
-            PB7<Alternate<AF7>>,
-            PB15<Alternate<AF4>>
+            gpio::PA10<Alternate<7>>,
+            gpio::PB7<Alternate<7>>,
+            gpio::PB15<Alternate<4>>
         ]
         CK: [
             NoCk,
-            PA8<Alternate<AF7>>
+            gpio::PA8<Alternate<7>>
         ]
     USART2:
         TX: [
             NoTx,
-            PA2<Alternate<AF7>>,
-            PD5<Alternate<AF7>>
+            gpio::PA2<Alternate<7>>,
+            gpio::PD5<Alternate<7>>
         ]
         RX: [
             NoRx,
-            PA3<Alternate<AF7>>,
-            PD6<Alternate<AF7>>
+            gpio::PA3<Alternate<7>>,
+            gpio::PD6<Alternate<7>>
         ]
         CK: [
             NoCk,
-            PA4<Alternate<AF7>>,
-            PD7<Alternate<AF7>>
+            gpio::PA4<Alternate<7>>,
+            gpio::PD7<Alternate<7>>
         ]
     USART3:
         TX: [
             NoTx,
-            PB10<Alternate<AF7>>,
-            PC10<Alternate<AF7>>,
-            PD8<Alternate<AF7>>
+            gpio::PB10<Alternate<7>>,
+            gpio::PC10<Alternate<7>>,
+            gpio::PD8<Alternate<7>>
         ]
         RX: [
             NoRx,
-            PB11<Alternate<AF7>>,
-            PC11<Alternate<AF7>>,
-            PD9<Alternate<AF7>>
+            gpio::PB11<Alternate<7>>,
+            gpio::PC11<Alternate<7>>,
+            gpio::PD9<Alternate<7>>
         ]
         CK: [
             NoCk,
-            PB12<Alternate<AF7>>,
-            PC12<Alternate<AF7>>,
-            PD10<Alternate<AF7>>
+            gpio::PB12<Alternate<7>>,
+            gpio::PC12<Alternate<7>>,
+            gpio::PD10<Alternate<7>>
         ]
     USART6:
         TX: [
             NoTx,
-            PC6<Alternate<AF7>>,
-            PG14<Alternate<AF7>>
+            gpio::PC6<Alternate<7>>,
+            gpio::PG14<Alternate<7>>
         ]
         RX: [
             NoRx,
-            PC7<Alternate<AF7>>,
-            PG9<Alternate<AF7>>
+            gpio::PC7<Alternate<7>>,
+            gpio::PG9<Alternate<7>>
         ]
         CK: [
             NoCk,
-            PC8<Alternate<AF7>>,
-            PG7<Alternate<AF7>>
+            gpio::PC8<Alternate<7>>,
+            gpio::PG7<Alternate<7>>
         ]
 }
 uart_pins! {
     UART4:
         TX: [
             NoTx,
-            PA0<Alternate<AF8>>,
-            PA12<Alternate<AF6>>,
-            PB9<Alternate<AF8>>,
-            PC10<Alternate<AF8>>,
-            PD1<Alternate<AF8>>,
-            PH13<Alternate<AF8>>
+            gpio::PA0<Alternate<8>>,
+            gpio::PA12<Alternate<6>>,
+            gpio::PB9<Alternate<8>>,
+            gpio::PC10<Alternate<8>>,
+            gpio::PD1<Alternate<8>>,
+            gpio::PH13<Alternate<8>>
         ]
         RX: [
             NoRx,
-            PA1<Alternate<AF8>>,
-            PA11<Alternate<AF6>>,
-            PB8<Alternate<AF8>>,
-            PC11<Alternate<AF8>>,
-            PD0<Alternate<AF8>>,
-            PH14<Alternate<AF8>>,
-            PI9<Alternate<AF8>>
+            gpio::PA1<Alternate<8>>,
+            gpio::PA11<Alternate<6>>,
+            gpio::PB8<Alternate<8>>,
+            gpio::PC11<Alternate<8>>,
+            gpio::PD0<Alternate<8>>,
+            gpio::PH14<Alternate<8>>,
+            #[cfg(not(feature = "rm0468"))]
+            gpio::PI9<Alternate<8>>
         ]
     UART5:
         TX: [
             NoTx,
-            PB6<Alternate<AF14>>,
-            PB13<Alternate<AF14>>,
-            PC12<Alternate<AF8>>
+            gpio::PB6<Alternate<14>>,
+            gpio::PB13<Alternate<14>>,
+            gpio::PC12<Alternate<8>>
         ]
         RX: [
             NoRx,
-            PB5<Alternate<AF14>>,
-            PB12<Alternate<AF14>>,
-            PD2<Alternate<AF8>>
+            gpio::PB5<Alternate<14>>,
+            gpio::PB12<Alternate<14>>,
+            gpio::PD2<Alternate<8>>
         ]
     UART7:
         TX: [
             NoTx,
-            PA15<Alternate<AF11>>,
-            PB4<Alternate<AF11>>,
-            PE8<Alternate<AF7>>,
-            PF7<Alternate<AF7>>
+            gpio::PA15<Alternate<11>>,
+            gpio::PB4<Alternate<11>>,
+            gpio::PE8<Alternate<7>>,
+            gpio::PF7<Alternate<7>>
         ]
         RX: [
             NoRx,
-            PA8<Alternate<AF11>>,
-            PB3<Alternate<AF11>>,
-            PE7<Alternate<AF7>>,
-            PF6<Alternate<AF7>>
+            gpio::PA8<Alternate<11>>,
+            gpio::PB3<Alternate<11>>,
+            gpio::PE7<Alternate<7>>,
+            gpio::PF6<Alternate<7>>
         ]
     UART8:
         TX: [
             NoTx,
-            PE1<Alternate<AF8>>,
+            gpio::PE1<Alternate<8>>,
             #[cfg(not(feature = "stm32h7b0"))]
-            PJ8<Alternate<AF8>>
+            gpio::PJ8<Alternate<8>>
         ]
         RX: [
             NoRx,
-            PE0<Alternate<AF8>>,
+            gpio::PE0<Alternate<8>>,
             #[cfg(not(feature = "stm32h7b0"))]
-            PJ9<Alternate<AF8>>
+            gpio::PJ9<Alternate<8>>
         ]
 }
 
 /// Serial abstraction
 pub struct Serial<USART> {
     pub(crate) usart: USART,
+    ker_ck: Hertz,
 }
 
 /// Serial receiver
 pub struct Rx<USART> {
     _usart: PhantomData<USART>,
+    ker_ck: Hertz,
 }
 
 /// Serial transmitter
@@ -368,9 +489,9 @@ pub struct Tx<USART> {
 pub trait SerialExt<USART>: Sized {
     type Rec: ResetEnable;
 
-    fn serial(
+    fn serial<P: Pins<USART>>(
         self,
-        _pins: impl Pins<USART>,
+        _pins: P,
         config: impl Into<config::Config>,
         prec: Self::Rec,
         clocks: &CoreClocks,
@@ -381,6 +502,7 @@ pub trait SerialExt<USART>: Sized {
         config: impl Into<config::Config>,
         prec: Self::Rec,
         clocks: &CoreClocks,
+        synchronous: bool,
     ) -> Result<Serial<USART>, config::InvalidConfig>;
 
     #[deprecated(since = "0.7.0", note = "Deprecated in favour of .serial(..)")]
@@ -403,14 +525,20 @@ pub trait SerialExt<USART>: Sized {
         config: impl Into<config::Config>,
         prec: Self::Rec,
         clocks: &CoreClocks,
+        synchronous: bool,
     ) -> Result<Serial<USART>, config::InvalidConfig> {
-        self.serial_unchecked(config, prec, clocks)
+        self.serial_unchecked(config, prec, clocks, synchronous)
     }
 }
 
+macro_rules! replace_expr {
+    ($_t:tt $sub:expr) => {
+        $sub
+    };
+}
 macro_rules! usart {
     ($(
-        $USARTX:ident: ($usartX:ident, $Rec:ident, $pclkX:ident),
+        $USARTX:ident: ($usartX:ident, $Rec:ident, $pclkX:ident $(, $synchronous:ident)?),
     )+) => {
         $(
             /// Configures a USART peripheral to provide serial
@@ -421,55 +549,130 @@ macro_rules! usart {
                     config: impl Into<config::Config>,
                     prec: rec::$Rec,
                     clocks: &CoreClocks
+                    $(, $synchronous: bool)?
                 ) -> Result<Self, config::InvalidConfig>
                 {
-                    use crate::stm32::usart1::cr2::STOP_A as STOP;
-                    use self::config::*;
-
-                    let config = config.into();
-
                     // Enable clock for USART and reset
                     prec.enable().reset();
 
-                    // Get kernel clock
-	                let usart_ker_ck = match Self::kernel_clk(clocks) {
-                        Some(ker_hz) => ker_hz.0,
-                        _ => panic!("$USARTX kernel clock not running!")
-                    };
+                    let ker_ck = Self::kernel_clk_unwrap(clocks);
+                    let mut serial = Serial { usart, ker_ck };
+                    let config = config.into();
+                    serial.usart.cr1.reset();
+                    serial.configure(&config $(, $synchronous )?);
+
+                    Ok(serial)
+                }
+
+                /// Runs the serial port configuration process
+                ///
+                /// The serial port must be disabled when called.
+                fn configure(&mut self, config: &config::Config $(, $synchronous: bool)?) {
+                    use crate::stm32::usart1::cr2::STOP_A as STOP;
+                    use self::config::*;
 
                     // Prescaler not used for now
-                    let usart_ker_ck_presc = usart_ker_ck;
-                    usart.presc.reset();
+                    let usart_ker_ck_presc = self.ker_ck;
+                    self.usart.presc.reset();
 
                     // Calculate baudrate divisor
-                    let usartdiv = usart_ker_ck_presc / config.baudrate.0;
+                    let usartdiv = usart_ker_ck_presc / config.baudrate;
                     assert!(usartdiv <= 65_536);
 
                     // 16 times oversampling, OVER8 = 0
                     let brr = usartdiv as u16;
-                    usart.brr.write(|w| { w.brr().bits(brr) });
-
-                    // disable hardware flow control
-                    // TODO enable DMA
-                    // usart.cr3.write(|w| w.rtse().clear_bit().ctse().clear_bit());
+                    self.usart.brr.write(|w| { w.brr().bits(brr) });
 
                     // Reset registers to disable advanced USART features
-                    usart.cr2.reset();
-                    usart.cr3.reset();
+                    self.usart.cr2.reset();
+                    self.usart.cr3.reset();
 
-                    // Set stop bits
-                    usart.cr2.write(|w| {
+                    // RXFIFO threshold
+                    let fifo_threshold_bits = match config.rxfifothreshold {
+                        FifoThreshold::Eighth => 0,
+                        FifoThreshold::Quarter => 1,
+                        FifoThreshold::Half => 2,
+                        FifoThreshold::ThreeQuarter => 3,
+                        FifoThreshold::SevenEighth => 4,
+                        FifoThreshold::Full => 5,
+                    };
+                    unsafe {
+                        self.usart.cr3.modify(|_, w| w.rxftcfg().bits(fifo_threshold_bits));
+                    }
+
+                    // TXFIFO threashold
+                    let fifo_threshold_bits = match config.txfifothreshold {
+                        FifoThreshold::Eighth => 0,
+                        FifoThreshold::Quarter => 1,
+                        FifoThreshold::Half => 2,
+                        FifoThreshold::ThreeQuarter => 3,
+                        FifoThreshold::SevenEighth => 4,
+                        FifoThreshold::Full => 5,
+                    };
+                    unsafe {
+                        self.usart.cr3.modify(|_, w| w.txftcfg().bits(fifo_threshold_bits));
+                    }
+
+                    // Configure serial mode
+                    self.usart.cr2.write(|w| {
                         w.stop().variant(match config.stopbits {
                             StopBits::STOP0P5 => STOP::STOP0P5,
                             StopBits::STOP1 => STOP::STOP1,
                             StopBits::STOP1P5 => STOP::STOP1P5,
                             StopBits::STOP2 => STOP::STOP2,
-                        })
+                        });
+
+                        w.msbfirst().variant(match config.bitorder {
+                            BitOrder::LsbFirst => MSBFIRST_A::LSB,
+                            BitOrder::MsbFirst => MSBFIRST_A::MSB,
+                        });
+
+                        w.swap().bit(config.swaptxrx);
+
+                        w.rxinv().variant(if config.invertrx {
+                            RXINV_A::INVERTED
+                        } else {
+                            RXINV_A::STANDARD
+                        });
+
+                        w.txinv().variant(if config.inverttx {
+                            TXINV_A::INVERTED
+                        } else {
+                            TXINV_A::STANDARD
+                        });
+
+                        // If synchronous mode is not supported, these bits are
+                        // reserved and must be kept at reset value
+                        $(
+                            w.lbcl().variant(if config.lastbitclockpulse {
+                                LBCL_A::OUTPUT
+                            } else {
+                                LBCL_A::NOTOUTPUT
+                            });
+
+                            w.clken().variant(if $synchronous {
+                                CLKEN_A::ENABLED
+                            } else {
+                                CLKEN_A::DISABLED
+                            });
+
+                            w.cpol().variant(match config.clockpolarity {
+                                ClockPolarity::IdleHigh =>CPOL_A::HIGH,
+                                ClockPolarity::IdleLow =>CPOL_A::LOW
+                            });
+
+                            w.cpha().variant(match config.clockphase {
+                                ClockPhase::First => CPHA_A::FIRST,
+                                ClockPhase::Second => CPHA_A::SECOND
+                            });
+                        )?
+
+                        w
                     });
 
-                    // Enable transmission and receiving
-                    // and configure frame
-                    usart.cr1.write(|w| {
+                    // Enable transmission and receiving and configure frame
+                    // Retain enabled events
+                    self.usart.cr1.modify(|_, w| {
                         w.fifoen()
                             .set_bit() // FIFO mode enabled
                             .over8()
@@ -483,21 +686,37 @@ macro_rules! usart {
                             .m1()
                             .clear_bit()
                             .m0()
-                            .variant(match config.wordlength {
-                                WordLength::DataBits8 => M0::BIT8,
-                                WordLength::DataBits9 => M0::BIT9,
+                            .variant(match config.parity {
+                                Parity::ParityNone => M0::BIT8,
+                                _ => M0::BIT9,
                             }).pce()
                             .variant(match config.parity {
                                 Parity::ParityNone => PCE::DISABLED,
                                 _ => PCE::ENABLED,
                             }).ps()
                             .variant(match config.parity {
-                                Parity::ParityOdd => PS::EVEN,
-                                _ => PS::ODD,
+                                Parity::ParityOdd => PS::ODD,
+                                _ => PS::EVEN,
                             })
                     });
+                }
 
-                    Ok(Serial { usart })
+                /// Applies the configuration to the serial port.
+                ///
+                /// Ensure that the serial port is not transmitting data when calling this method.
+                ///
+                /// # Panics
+                ///
+                /// Panics if DMA Rx or Tx are enabled.
+                pub fn reconfigure(&mut self, config: impl Into<config::Config> $(, $synchronous: bool)?) {
+                    if self.dma_rx_enabled() || self.dma_tx_enabled() {
+                        panic!("Cannot reconfigure serial while DMA enabled");
+                    }
+
+                    self.usart.cr1.modify(|_, w| w.ue().disabled());
+
+                    let config = config.into();
+                    self.configure(&config $(, $synchronous )?);
                 }
 
                 /// Enables the Rx DMA stream.
@@ -510,6 +729,11 @@ macro_rules! usart {
                     self.usart.cr3.modify(|_, w| w.dmar().clear_bit());
                 }
 
+                /// Returns `true` if the Rx DMA stream is enabled.
+                pub fn dma_rx_enabled(&self) -> bool {
+                    self.usart.cr3.read().dmar().bit_is_set()
+                }
+
                 /// Enables the Tx DMA stream.
                 pub fn enable_dma_tx(&mut self) {
                     self.usart.cr3.modify(|_, w| w.dmat().set_bit());
@@ -520,6 +744,11 @@ macro_rules! usart {
                     self.usart.cr3.modify(|_, w| w.dmat().clear_bit());
                 }
 
+                /// Returns `true` if the Tx DMA stream is enabled.
+                pub fn dma_tx_enabled(&self) -> bool {
+                    self.usart.cr3.read().dmat().bit_is_set()
+                }
+
                 /// Starts listening for an interrupt event
                 pub fn listen(&mut self, event: Event) {
                     match event {
@@ -532,6 +761,12 @@ macro_rules! usart {
                         Event::Idle => {
                             self.usart.cr1.modify(|_, w| w.idleie().enabled())
                         },
+                        Event::Txftie => {
+                            self.usart.cr3.modify(|_, w| w.txftie().set_bit())
+                        },
+                        Event::Rxftie => {
+                            self.usart.cr3.modify(|_, w| w.rxftie().set_bit())
+                        },
                     }
                 }
 
@@ -547,6 +782,12 @@ macro_rules! usart {
                         Event::Idle => {
                             self.usart.cr1.modify(|_, w| w.idleie().disabled())
                         },
+                        Event::Txftie => {
+                            self.usart.cr3.modify(|_, w| w.txftie().clear_bit())
+                        },
+                        Event::Rxftie => {
+                            self.usart.cr3.modify(|_, w| w.rxftie().clear_bit())
+                        },
                     }
                     let _ = self.usart.cr1.read();
                     let _ = self.usart.cr1.read(); // Delay 2 peripheral clocks
@@ -585,6 +826,8 @@ macro_rules! usart {
                     unsafe { (*$USARTX::ptr()).isr.read().rxne().bit_is_set() }
                 }
 
+                /// Splits the [`Serial`] struct into transmit ([`Tx`]) and receive ([`Rx`]) parts which can be used
+                /// separately.
                 pub fn split(self) -> (Tx<$USARTX>, Rx<$USARTX>) {
                     (
                         Tx {
@@ -592,9 +835,21 @@ macro_rules! usart {
                         },
                         Rx {
                             _usart: PhantomData,
+                            ker_ck: self.ker_ck,
                         },
                     )
                 }
+
+                /// Combines the [`Tx`] and [`Rx`] structs from [`Serial::split()`] into a [`Serial`]
+                #[allow(unused_variables)]
+                pub fn join(tx: Tx<$USARTX>, rx: Rx<$USARTX>) -> Self {
+                    assert_eq!(core::mem::size_of::<$USARTX>(), 0);
+                    Self {
+                        usart: unsafe { core::mem::zeroed::<$USARTX>() },
+                        ker_ck: rx.ker_ck,
+                    }
+                }
+
                 /// Releases the USART peripheral
                 pub fn release(self) -> $USARTX {
                     // Wait until both TXFIFO and shift register are empty
@@ -602,28 +857,46 @@ macro_rules! usart {
 
                     self.usart
                 }
+
+                /// Returns a reference to the inner peripheral
+                pub fn inner(&self) -> &$USARTX {
+                    &self.usart
+                }
+
+                /// Returns a mutable reference to the inner peripheral
+                pub fn inner_mut(&mut self) -> &mut $USARTX {
+                    &mut self.usart
+                }
             }
 
             impl SerialExt<$USARTX> for $USARTX {
                 type Rec = rec::$Rec;
 
-                fn serial(self,
-                         _pins: impl Pins<$USARTX>,
+                fn serial<P: Pins<$USARTX>>(self,
+                         _pins: P,
                          config: impl Into<config::Config>,
                          prec: rec::$Rec,
                          clocks: &CoreClocks
                 ) -> Result<Serial<$USARTX>, config::InvalidConfig>
                 {
-                    Serial::$usartX(self, config, prec, clocks)
+                    Serial::$usartX(
+                        self, config, prec, clocks
+                            $(, replace_expr!($synchronous P::SYNCHRONOUS))?
+                    )
                 }
 
                 fn serial_unchecked(self,
                                    config: impl Into<config::Config>,
                                    prec: rec::$Rec,
-                                   clocks: &CoreClocks
+                                   clocks: &CoreClocks,
+                                   #[allow(unused)]
+                                   synchronous: bool,
                 ) -> Result<Serial<$USARTX>, config::InvalidConfig>
                 {
-                    Serial::$usartX(self, config, prec, clocks)
+                    Serial::$usartX(
+                        self, config, prec, clocks
+                            $(, replace_expr!($synchronous synchronous))?
+                    )
                 }
             }
 
@@ -633,6 +906,7 @@ macro_rules! usart {
                 fn read(&mut self) -> nb::Result<u8, Error> {
                     let mut rx: Rx<$USARTX> = Rx {
                         _usart: PhantomData,
+                        ker_ck: self.ker_ck,
                     };
                     rx.read()
                 }
@@ -727,16 +1001,16 @@ macro_rules! usart {
             }
 
             impl serial::Write<u8> for Serial<$USARTX> {
-                type Error = Never;
+                type Error = core::convert::Infallible;
 
-                fn flush(&mut self) -> nb::Result<(), Never> {
+                fn flush(&mut self) -> nb::Result<(), Self::Error> {
                     let mut tx: Tx<$USARTX> = Tx {
                         _usart: PhantomData,
                     };
                     tx.flush()
                 }
 
-                fn write(&mut self, byte: u8) -> nb::Result<(), Never> {
+                fn write(&mut self, byte: u8) -> nb::Result<(), Self::Error> {
                     let mut tx: Tx<$USARTX> = Tx {
                         _usart: PhantomData,
                     };
@@ -755,9 +1029,9 @@ macro_rules! usart {
                 // framing errors (which only occur in SmartCard
                 // mode); neither of these apply to our hardware
                 // configuration
-                type Error = Never;
+                type Error = core::convert::Infallible;
 
-                fn flush(&mut self) -> nb::Result<(), Never> {
+                fn flush(&mut self) -> nb::Result<(), Self::Error> {
                     // NOTE(unsafe) atomic read with no side effects
                     let isr = unsafe { (*$USARTX::ptr()).isr.read() };
 
@@ -768,7 +1042,7 @@ macro_rules! usart {
                     }
                 }
 
-                fn write(&mut self, byte: u8) -> nb::Result<(), Never> {
+                fn write(&mut self, byte: u8) -> nb::Result<(), Self::Error> {
                     // NOTE(unsafe) atomic read with no side effects
                     let isr = unsafe { (*$USARTX::ptr()).isr.read() };
 
@@ -831,17 +1105,53 @@ macro_rules! usart_sel {
             impl Serial<$USARTX> {
                 /// Returns the frequency of the current kernel clock for
                 #[doc=$doc]
-                fn kernel_clk(clocks: &CoreClocks) -> Option<Hertz> {
+                pub fn kernel_clk(clocks: &CoreClocks) -> Option<Hertz> {
                     // unsafe: read only
                     let ccip = unsafe { (*stm32::RCC::ptr()).$ccip.read() };
 
                     match ccip.$sel().variant() {
-                        Val($SEL::$PCLK) => Some(clocks.$pclk()),
-                        Val($SEL::PLL2_Q) => clocks.pll2_q_ck(),
-                        Val($SEL::PLL3_Q) => clocks.pll3_q_ck(),
-                        Val($SEL::HSI_KER) => clocks.hsi_ck(),
-                        Val($SEL::CSI_KER) => clocks.csi_ck(),
-                        Val($SEL::LSE) => unimplemented!(),
+                        Some($SEL::$PCLK) => Some(clocks.$pclk()),
+                        Some($SEL::PLL2_Q) => clocks.pll2_q_ck(),
+                        Some($SEL::PLL3_Q) => clocks.pll3_q_ck(),
+                        Some($SEL::HSI_KER) => clocks.hsi_ck(),
+                        Some($SEL::CSI_KER) => clocks.csi_ck(),
+                        Some($SEL::LSE) => unimplemented!(),
+                        _ => unreachable!(),
+                    }
+                }
+                /// Returns the frequency of the current kernel clock for
+                #[doc=$doc]
+                ///
+                /// # Panics
+                ///
+                /// Panics if the kernel clock is not running
+                pub fn kernel_clk_unwrap(clocks: &CoreClocks) -> Hertz {
+                    // unsafe: read only
+                    let ccip = unsafe { (*stm32::RCC::ptr()).$ccip.read() };
+
+                    match ccip.$sel().variant() {
+                        Some($SEL::$PCLK) => clocks.$pclk(),
+                        Some($SEL::PLL2_Q) => {
+                            clocks.pll2_q_ck().expect(
+                                concat!(stringify!($USARTX), ": PLL2_Q must be enabled")
+                            )
+                        }
+                        Some($SEL::PLL3_Q) => {
+                            clocks.pll3_q_ck().expect(
+                                concat!(stringify!($USARTX), ": PLL3_Q must be enabled")
+                            )
+                        }
+                        Some($SEL::HSI_KER) => {
+                            clocks.hsi_ck().expect(
+                                concat!(stringify!($USARTX), ": HSI clock must be enabled")
+                            )
+                        }
+                        Some($SEL::CSI_KER) => {
+                            clocks.csi_ck().expect(
+                                concat!(stringify!($USARTX), ": CSI clock must be enabled")
+                            )
+                        }
+                        Some($SEL::LSE) => unimplemented!(),
                         _ => unreachable!(),
                     }
                 }
@@ -851,10 +1161,10 @@ macro_rules! usart_sel {
 }
 
 usart! {
-    USART1: (usart1, Usart1, pclk2),
-    USART2: (usart2, Usart2, pclk1),
-    USART3: (usart3, Usart3, pclk1),
-    USART6: (usart6, Usart6, pclk2),
+    USART1: (usart1, Usart1, pclk2, synchronous),
+    USART2: (usart2, Usart2, pclk1, synchronous),
+    USART3: (usart3, Usart3, pclk1, synchronous),
+    USART6: (usart6, Usart6, pclk2, synchronous),
 
     UART4: (uart4, Uart4, pclk1),
     UART5: (uart5, Uart5, pclk1),
@@ -862,7 +1172,7 @@ usart! {
     UART8: (uart8, Uart8, pclk1),
 }
 
-#[cfg(not(feature = "rm0455"))]
+#[cfg(any(feature = "rm0433", feature = "rm0399"))]
 usart_sel! {
     d2ccip2r, USART16SEL_A, usart16sel, RCC_PCLK2, pclk2;
 
@@ -876,6 +1186,13 @@ usart_sel! {
     USART1: "USART1",
     USART6: "USART6",
 }
+#[cfg(feature = "rm0468")]
+usart_sel! {
+    d2ccip2r, USART16910SEL_A, usart16910sel, RCC_PCLK2, pclk2;
+
+    USART1: "USART1",
+    USART6: "USART6",
+}
 
 #[cfg(not(feature = "rm0455"))]
 usart_sel! {
