@@ -1,4 +1,9 @@
 //! Timers
+//!
+//! # Examples
+//!
+//! - [Blinky using a Timer](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/blinky_timer.rs)
+//! - [64 bit microsecond timer](https://github.com/stm32-rs/stm32h7xx-hal/blob/master/examples/tick_timer.rs)
 
 // TODO: on the h7x3 at least, only TIM2, TIM3, TIM4, TIM5 can support 32 bits.
 // TIM1 is 16 bit.
@@ -29,7 +34,6 @@ use crate::stm32::rcc::{d2ccip2r as ccip2r, d3ccipr as srdccipr};
 use crate::rcc::{rec, CoreClocks, ResetEnable};
 use crate::stm32;
 use crate::time::Hertz;
-use stm32h7::Variant::Val;
 
 /// Associate clocks with timers
 pub trait GetClk {
@@ -66,12 +70,12 @@ impl GetClk for LPTIM1 {
         let ccip2r = &unsafe { &*stm32::RCC::ptr() }.cdccip2r;
 
         match ccip2r.read().lptim1sel().variant() {
-            Val(ccip2r::LPTIM1SEL_A::RCC_PCLK1) => Some(clocks.pclk1()),
-            Val(ccip2r::LPTIM1SEL_A::PLL2_P) => clocks.pll2_p_ck(),
-            Val(ccip2r::LPTIM1SEL_A::PLL3_R) => clocks.pll3_r_ck(),
-            Val(ccip2r::LPTIM1SEL_A::LSE) => unimplemented!(),
-            Val(ccip2r::LPTIM1SEL_A::LSI) => unimplemented!(),
-            Val(ccip2r::LPTIM1SEL_A::PER) => clocks.per_ck(),
+            Some(ccip2r::LPTIM1SEL_A::RCC_PCLK1) => Some(clocks.pclk1()),
+            Some(ccip2r::LPTIM1SEL_A::PLL2_P) => clocks.pll2_p_ck(),
+            Some(ccip2r::LPTIM1SEL_A::PLL3_R) => clocks.pll3_r_ck(),
+            Some(ccip2r::LPTIM1SEL_A::LSE) => unimplemented!(),
+            Some(ccip2r::LPTIM1SEL_A::LSI) => unimplemented!(),
+            Some(ccip2r::LPTIM1SEL_A::PER) => clocks.per_ck(),
             _ => unreachable!(),
         }
     }
@@ -87,12 +91,12 @@ impl GetClk for LPTIM2 {
         let srdccipr = &unsafe { &*stm32::RCC::ptr() }.srdccipr;
 
         match srdccipr.read().lptim2sel().variant() {
-            Val(srdccipr::LPTIM2SEL_A::RCC_PCLK4) => Some(clocks.pclk4()),
-            Val(srdccipr::LPTIM2SEL_A::PLL2_P) => clocks.pll2_p_ck(),
-            Val(srdccipr::LPTIM2SEL_A::PLL3_R) => clocks.pll3_r_ck(),
-            Val(srdccipr::LPTIM2SEL_A::LSE) => unimplemented!(),
-            Val(srdccipr::LPTIM2SEL_A::LSI) => unimplemented!(),
-            Val(srdccipr::LPTIM2SEL_A::PER) => clocks.per_ck(),
+            Some(srdccipr::LPTIM2SEL_A::RCC_PCLK4) => Some(clocks.pclk4()),
+            Some(srdccipr::LPTIM2SEL_A::PLL2_P) => clocks.pll2_p_ck(),
+            Some(srdccipr::LPTIM2SEL_A::PLL3_R) => clocks.pll3_r_ck(),
+            Some(srdccipr::LPTIM2SEL_A::LSE) => unimplemented!(),
+            Some(srdccipr::LPTIM2SEL_A::LSI) => unimplemented!(),
+            Some(srdccipr::LPTIM2SEL_A::PER) => clocks.per_ck(),
             _ => unreachable!(),
         }
     }
@@ -130,12 +134,12 @@ macro_rules! impl_clk_lptim345 {
                     let d3ccipr = &unsafe { &*stm32::RCC::ptr() }.d3ccipr;
 
                     match d3ccipr.read().lptim345sel().variant() {
-                        Val(srdccipr::LPTIM345SEL_A::RCC_PCLK4) => Some(clocks.pclk4()),
-                        Val(srdccipr::LPTIM345SEL_A::PLL2_P) => clocks.pll2_p_ck(),
-                        Val(srdccipr::LPTIM345SEL_A::PLL3_R) => clocks.pll3_r_ck(),
-                        Val(srdccipr::LPTIM345SEL_A::LSE) => unimplemented!(),
-                        Val(srdccipr::LPTIM345SEL_A::LSI) => unimplemented!(),
-                        Val(srdccipr::LPTIM345SEL_A::PER) => clocks.per_ck(),
+                        Some(srdccipr::LPTIM345SEL_A::RCC_PCLK4) => Some(clocks.pclk4()),
+                        Some(srdccipr::LPTIM345SEL_A::PLL2_P) => clocks.pll2_p_ck(),
+                        Some(srdccipr::LPTIM345SEL_A::PLL3_R) => clocks.pll3_r_ck(),
+                        Some(srdccipr::LPTIM345SEL_A::LSE) => unimplemented!(),
+                        Some(srdccipr::LPTIM345SEL_A::LSI) => unimplemented!(),
+                        Some(srdccipr::LPTIM345SEL_A::PER) => clocks.per_ck(),
                         _ => unreachable!(),
                     }
                 }
@@ -158,27 +162,29 @@ pub trait TimerExt<TIM> {
     /// Configures a periodic timer
     ///
     /// Generates an overflow event at the `timeout` frequency.
-    fn timer<T>(self, timeout: T, prec: Self::Rec, clocks: &CoreClocks) -> TIM
-    where
-        T: Into<Hertz>;
+    fn timer(self, timeout: Hertz, prec: Self::Rec, clocks: &CoreClocks)
+        -> TIM;
 
     /// Configures the timer to count up at the given frequency
     ///
     /// Counts from 0 to the counter's maximum value, then repeats.
     /// Because this only uses the timer prescaler, the frequency
     /// is rounded to a multiple of the timer's kernel clock.
-    fn tick_timer<T>(
+    ///
+    /// For example, calling `.tick_timer(1.MHz(), ..)` for a 16-bit timer will
+    /// result in a timers that increments every microsecond and overflows every
+    /// ~65 milliseconds
+    fn tick_timer(
         self,
-        frequency: T,
+        frequency: Hertz,
         prec: Self::Rec,
         clocks: &CoreClocks,
-    ) -> TIM
-    where
-        T: Into<Hertz>;
+    ) -> TIM;
 }
 
 /// Hardware timers
 #[derive(Debug)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub struct Timer<TIM> {
     clk: u32,
     tim: TIM,
@@ -224,7 +230,7 @@ macro_rules! hal {
                     self.clear_irq();
 
                     // Set PSC and ARR
-                    self.set_freq(timeout);
+                    self.set_freq(timeout.into());
 
                     // Generate an update event to force an update of the ARR register. This ensures
                     // the first timer cycle is of the specified duration.
@@ -247,23 +253,17 @@ macro_rules! hal {
             impl TimerExt<Timer<$TIMX>> for $TIMX {
                 type Rec = rec::$Rec;
 
-                fn timer<T>(self, timeout: T,
+                fn timer(self, timeout: Hertz,
                             prec: Self::Rec, clocks: &CoreClocks
-                ) -> Timer<$TIMX>
-                where
-                    T: Into<Hertz>,
-                {
+                ) -> Timer<$TIMX> {
                     let mut timer = Timer::$timX(self, prec, clocks);
                     timer.start(timeout);
                     timer
                 }
 
-                fn tick_timer<T>(self, frequency: T,
+                fn tick_timer(self, frequency: Hertz,
                                  prec: Self::Rec, clocks: &CoreClocks
-                ) -> Timer<$TIMX>
-                where
-                    T: Into<Hertz>,
-                {
+                ) -> Timer<$TIMX> {
                     let mut timer = Timer::$timX(self, prec, clocks);
 
                     timer.pause();
@@ -293,10 +293,10 @@ macro_rules! hal {
                 pub fn $timX(tim: $TIMX, prec: rec::$Rec, clocks: &CoreClocks) -> Self
                 {
                     // enable and reset peripheral to a clean state
-                    prec.enable().reset();
+                    let _ = prec.enable().reset(); // drop, can be recreated by free method
 
                     let clk = $TIMX::get_clk(clocks)
-                        .expect("Timer input clock not running!").0;
+                        .expect(concat!(stringify!($TIMX), ": Input clock not running!")).raw();
 
                     Timer {
                         clk,
@@ -306,12 +306,8 @@ macro_rules! hal {
 
                 /// Configures the timer's frequency and counter reload value
                 /// so that it underflows at the timeout's frequency
-                pub fn set_freq<T>(&mut self, timeout: T)
-                where
-                    T: Into<Hertz>,
-                {
-                    let timeout = timeout.into();
-                    let ticks = self.clk / timeout.0;
+                pub fn set_freq(&mut self, timeout: Hertz) {
+                    let ticks = self.clk / timeout.raw();
 
                     self.set_timeout_ticks(ticks);
                 }
@@ -319,8 +315,11 @@ macro_rules! hal {
                 /// Sets the timer period from a time duration
                 ///
                 /// ```
+                /// use stm32h7xx_hal::time::MilliSeconds;
+                ///
                 /// // Set timeout to 100ms
-                /// timer.set_timeout(100.ms());
+                /// let timeout = MilliSeconds::from_ticks(100).into_rate();
+                /// timer.set_timeout(timeout);
                 /// ```
                 ///
                 /// Alternatively, the duration can be set using the
@@ -373,12 +372,8 @@ macro_rules! hal {
                 /// Counts from 0 to the counter's maximum value, then repeats.
                 /// Because this only uses the timer prescaler, the frequency
                 /// is rounded to a multiple of the timer's kernel clock.
-                pub fn set_tick_freq<T>(&mut self, frequency: T)
-                where
-                    T: Into<Hertz>,
-                {
-                    let frequency = frequency.into();
-                    let div = self.clk / frequency.0;
+                pub fn set_tick_freq(&mut self, frequency: Hertz) {
+                    let div = self.clk / frequency.raw();
 
                     let psc = u16(div - 1).unwrap();
                     self.tim.psc.write(|w| w.psc().bits(psc));
@@ -418,7 +413,7 @@ macro_rules! hal {
 
                 /// Read the counter of the TIM peripheral
                 pub fn counter(&self) -> u32 {
-                    self.tim.cnt.read().bits()
+                    self.tim.cnt.read().cnt().bits().into()
                 }
 
                 /// Start listening for `event`
@@ -465,6 +460,16 @@ macro_rules! hal {
 
                     (self.tim, rec::$Rec { _marker: PhantomData })
                 }
+
+                /// Returns a reference to the inner peripheral
+                pub fn inner(&self) -> &$TIMX {
+                    &self.tim
+                }
+
+                /// Returns a mutable reference to the inner peripheral
+                pub fn inner_mut(&mut self) -> &mut $TIMX {
+                    &mut self.tim
+                }
             }
         )+
     }
@@ -534,7 +539,7 @@ macro_rules! lptim_hal {
                     self.tim.cr.write(|w| w.enable().disabled());
 
                     // Set prescale and ARR
-                    self.priv_set_freq(timeout); // side effect: enables counter
+                    self.priv_set_freq(timeout.into()); // side effect: enables counter
 
                     // Clear IRQ
                     self.clear_irq();
@@ -556,43 +561,65 @@ macro_rules! lptim_hal {
             impl TimerExt<LpTimer<$TIMX, Enabled>> for $TIMX {
                 type Rec = rec::$Rec;
 
-                fn timer<T>(self, timeout: T,
+                fn timer(self, timeout: Hertz,
                             prec: Self::Rec, clocks: &CoreClocks
-                ) -> LpTimer<$TIMX, Enabled>
-                    where
-                        T: Into<Hertz>,
-                {
+                ) -> LpTimer<$TIMX, Enabled> {
                     LpTimer::$timx(self, timeout, prec, clocks)
                 }
 
-                fn tick_timer<T>(self, _frequency: T,
-                                 _prec: Self::Rec, _clocks: &CoreClocks
+                fn tick_timer(self, frequency: Hertz,
+                                 prec: Self::Rec, clocks: &CoreClocks
                 ) -> LpTimer<$TIMX, Enabled>
-                where
-                    T: Into<Hertz>,
                 {
-                    unimplemented!()
+                    // enable and reset peripheral to a clean state
+                    let _ = prec.enable().reset(); // drop, can be recreated by free method
+
+                    let clk = $TIMX::get_clk(clocks)
+                        .expect(concat!(stringify!($TIMX), ": Input clock not running!")).raw();
+
+                    let mut timer = LpTimer {
+                        clk,
+                        tim: self,
+                        timeout: Hertz::from_raw(0),
+                        _enabled: PhantomData,
+                    };
+
+                    // Configures the timer to count up at the given frequency
+
+                    // Reset: counter must be running
+                    timer.reset_counter();
+
+                    // Disable counter
+                    timer.tim.cr.write(|w| w.enable().disabled());
+
+                    // Set tick frequency
+                    timer.priv_set_tick_freq(frequency);
+
+                    // Clear IRQ
+                    timer.clear_irq();
+
+                    // Start counter
+                    timer.tim.cr.write(|w| w.cntstrt().set_bit().enable().enabled());
+
+                    timer
                 }
             }
 
             impl LpTimer<$TIMX, Enabled> {
                 /// Configures a LPTIM peripheral as a periodic count down timer
-                pub fn $timx<T>(tim: $TIMX, timeout: T,
+                pub fn $timx(tim: $TIMX, timeout: Hertz,
                                 prec: rec::$Rec, clocks: &CoreClocks
-                ) -> Self
-                where
-                    T: Into<Hertz>,
-                {
+                ) -> Self {
                     // enable and reset peripheral to a clean state
-                    prec.enable().reset();
+                    let _ = prec.enable().reset(); // drop, can be recreated by free method
 
                     let clk = $TIMX::get_clk(clocks)
-                        .expect("Timer input clock not running!").0;
+                        .expect(concat!(stringify!($TIMX), ": Input clock not running!")).raw();
 
                     let mut timer = LpTimer {
                         clk,
                         tim,
-                        timeout: Hertz(0),
+                        timeout: Hertz::from_raw(0),
                         _enabled: PhantomData,
                     };
                     timer.start(timeout);
@@ -624,19 +651,27 @@ macro_rules! lptim_hal {
             }
 
             impl LpTimer<$TIMX, Disabled> {
-                /// Sets the frequency of the LPTIM counter.
+                /// Sets the frequency of the LPTIM counter
                 ///
-                /// The counter must be disabled.
-                pub fn set_freq<T>(&mut self, timeout: T)
-                where
-                    T: Into<Hertz>,
-                {
+                /// The counter must be disabled
+                pub fn set_freq(&mut self, timeout: Hertz) {
                     self.priv_set_freq(timeout); // side effect: enables counter
 
                     // Disable timer
                     self.tim.cr.write(|w| w.enable().disabled());
                 }
 
+                /// Configures the timer to count up at the given frequency
+                ///
+                /// The counter must be disabled
+                pub fn set_tick_freq(&mut self, frequency: Hertz)
+                {
+                    self.priv_set_tick_freq(frequency); // side effect: enables counter
+
+                    // Disable timer
+                    self.tim.cr.write(|w| w.enable().disabled());
+                }
+
                 /// Start listening for `event`
                 ///
                 /// The timer must be disabled, see RM0433 Rev 7. 43.4.13
@@ -682,14 +717,11 @@ macro_rules! lptim_hal {
                 /// Private method to set the frequency of the LPTIM
                 /// counter. The counter must be disabled, but it will be
                 /// enabled at the end of this method.
-                fn priv_set_freq<T>(&mut self, timeout: T)
-                where
-                    T: Into<Hertz>,
-                {
-                    self.timeout = timeout.into();
+                fn priv_set_freq(&mut self, timeout: Hertz) {
+                    self.timeout = timeout;
 
                     let clk = self.clk;
-                    let frequency = self.timeout.0;
+                    let frequency = self.timeout.raw();
                     let ticks = clk / frequency;
                     assert!(ticks < 128 * (1 << 16));
 
@@ -722,14 +754,52 @@ macro_rules! lptim_hal {
                     self.tim.icr.write(|w| w.arrokcf().clear());
                 }
 
+                /// Private method to configure the timer to count up at the
+                /// given frequency
+                ///
+                /// The counter must be disabled, but it will be enabled at the
+                /// end of this method
+                fn priv_set_tick_freq(&mut self, frequency: Hertz)
+                {
+                    // Calculate prescaler
+                    let frequency = frequency.raw();
+                    let ticks = (self.clk + frequency - 1) / frequency;
+                    assert!(ticks <= 128,
+                            "LPTIM input clock is too slow to achieve this frequency");
+
+                    let (prescale, _prescale_div) = match ticks {
+                        0..=1 => ($timXpac::cfgr::PRESC_A::DIV1, 1),
+                        2 => ($timXpac::cfgr::PRESC_A::DIV2, 2),
+                        3..=4 => ($timXpac::cfgr::PRESC_A::DIV4, 4),
+                        5..=8 => ($timXpac::cfgr::PRESC_A::DIV8, 8),
+                        9..=16 => ($timXpac::cfgr::PRESC_A::DIV16, 16),
+                        17..=32 => ($timXpac::cfgr::PRESC_A::DIV32, 32),
+                        33..=64 => ($timXpac::cfgr::PRESC_A::DIV64, 64),
+                        _ => ($timXpac::cfgr::PRESC_A::DIV128, 128),
+                    };
+
+                    // Write CFGR: LPTIM must be disabled
+                    self.tim.cfgr.modify(|_, w| w.presc().variant(prescale));
+
+                    // Enable
+                    self.tim.cr.write(|w| w.enable().enabled());
+
+                    // Set ARR = max
+
+                    // Write ARR: LPTIM must be enabled
+                    self.tim.arr.write(|w| w.arr().bits(0xFFFF as u16));
+                    while self.tim.isr.read().arrok().bit_is_clear() {}
+                    self.tim.icr.write(|w| w.arrokcf().clear());
+                }
+
                 /// Read the counter of the LPTIM peripheral
-                pub fn counter(&self) -> u32 {
+                pub fn counter(&self) -> u16 {
                     loop {
                         // Read once
-                        let count1 = self.tim.cnt.read().bits();
+                        let count1 = self.tim.cnt.read().cnt().bits();
 
                         // Read twice - see RM0433 Rev 7. 43.4.14
-                        let count2 = self.tim.cnt.read().bits();
+                        let count2 = self.tim.cnt.read().cnt().bits();
 
                         if count1 == count2 { return count2; }
                     }
